#! /usr/bin/perl -w
use strict;
use utf8;

#############################################
#
#	file:	SeparateYomi.pl
#		for shell perl
#
#	project:	JTeXEdit
#
#	Copyright (c) 1999-2002 Norihito KOISO
#	All Rights Reserved
#
#############################################
#
#	使用法：
#		"***.idxd" の単語と読みを / で区切って与えれば
#		分解した表を "***.idxd2" に出力する．
#		かなのみは省略される．
#
#############################################
#
#	履歴
#	2011-04-03: use Foundation
#	2008-06-05: perl UTF-8 環境 に対応
#	2003-06-15: 初版
#
#############################################

#use encoding 'shift_jis'; 
use open IO => ':encoding(shift_jis)';
use Foundation;
# man PerlObjCBridge

sub SetFileTypes($) {
	my ($path) = @_;
	my $sign = NSNumber->numberWithInt_(unpack("N", 'jtxE'));
	my $type = NSNumber->numberWithInt_(unpack("N", 'TEXT'));
	my $dict = NSMutableDictionary->dictionary;
	$dict->setObject_forKey_($sign, "NSFileHFSCreatorCode");
	$dict->setObject_forKey_($type, "NSFileHFSTypeCode");
	NSFileManager->defaultManager
		->changeFileAttributes_atPath_($dict, $path);
}

my %Dict = ();
my $DictIsDirty = '';
sub ReadDict() {
	$DictIsDirty = '';
	%Dict = ();
	while (defined(my $line = <IN>)) {
		chomp $line;
		my @items = split(/\n|\r/, $line);
		for my $item (@items) {
			next if ($item =~ /^%/ or $item eq '');
			my @content = split(/\t/, $item);
			if (@content == 2) {
				$Dict{$content[1]} = $content[0];
			} else {
				warn "辞書の形式が正しくありません:\n$item\n";
			}
		}
	}
}
sub WriteDict() {
	my %buff = ();
	while (my ($text, $yomi) = each %Dict) {
		$buff{"$yomi\t$text"} = 1;
	}
	for my $item (sort keys %buff) {
		print OUT "$item\n";
	}
}

sub OnlyKana($) {
	my ($str) = @_;
	# ASCII or ひらがな or カタカナ
	return $str =~ /^[ -~ァ-ヶー－ぁ-ゖー－]*$/;
}

sub SeparateWords() {
	my %hash = ();
	for my $keyline (keys %Dict) {
		my $valline = $Dict{$keyline};
		my @keys = split('/', $keyline);
		my @vals = split('/', $valline);
		die "'$keyline' => '$valline' の区分の数が合いません．\n"
			unless (@keys == @vals);
		for (my $i = 0; $i <= $#keys; ++$i) {
			my $key = $keys[$i];
			next if OnlyKana($key);
			my $val = $vals[$i];
			my $oldval = $hash{$key};
			if (defined($oldval)) {
				die "異なる読みです．['$key' => '$oldval', '$val']\n"
					unless ($val eq $oldval);
			} else {
				$hash{$key} = $val;
			}
		}
	}
	return %hash;
}

die ".idxd ファイルを指定して下さい．\n" unless @ARGV;
{
	my $file = shift;
	open(IN, $file) or die "辞書が開けません\n";
	ReadDict();
	close(IN);
	%Dict = SeparateWords();
	my $out = $file.'2';
	open(OUT, ">$out") or die "ファイルが開けません:\n'$out'\n";
	WriteDict();
	close OUT;
	SetFileTypes($out);
}
print "終了．\n";

exit 0;
