#!/usr/bin/perl -w
use strict;
use utf8;
use open IO => ":encoding(iso-latin-1)";
use Encode;

# 使用法:
#	2 つのファイルまたはフォルダを比較する．
#	一度に渡せないときは "指令/順序処理..." すること．

my @TryEncodings = ('shiftjis', 'utf8', 'euc-jp', 'iso-Latin-1');
	# used in TryDecode
	# +? 'iso-2022-jp' is 7-bit encoding. difficult to detect

my ($Obj1, $Obj2) = @ARGV;
utf8::decode($Obj1);
utf8::decode($Obj2);
die "Error: Pass two objects.\n" unless defined($Obj2);
my $DirMode = -d $Obj1 or -d $Obj2;
print "/usr/bin/diff '$Obj1' '$Obj2'\n";

sub NewFile($) {
	my ($path) = @_;
	my $tmp = 0;
	while (-e "${path}_$tmp") { ++$tmp; }
	return "${path}_$tmp";
}

sub NewLine($) {
	my $file = shift;
	open(IN, "<$file") or die "$!: '$file'";
	my $text;
	sysread(IN, $text, 1024);
	if ($text =~ /\n/) {
		close IN;
		return $file;
	}
	close IN;
	open(IN, "<$file");
	$text = join('', <IN>);
	close IN;
	$text =~ tr/\r/\n/;
	my $tmp_path = NewFile($file);
	open(OUT, ">$tmp_path") or die "$file => '$tmp_path': $! ";
	print OUT $text;
	close OUT;
	return $tmp_path;
}

my %Table;
my @SameName;
sub AddFiles($);
sub AddFiles($) {
	my ($path) = @_;
	if (-d $path) {
		opendir(DIR, $path) or die $!;
		my @items = readdir(DIR);
		for (@items) { utf8::decode($_); }
		closedir(DIR);
		for my $item (@items) {
			next if ($item =~ /^\./);
			next if (-l "$path/$item");
			AddFiles("$path/$item");
		}
	} else {
		$path =~ m|/([^/]*)$|;
		if (defined($Table{$1})) {
			push(@SameName, $path);
		} else {
			$Table{$1} = $path;
		}
	}
}

my %Sames = ();
sub MakeTable($) {
	my ($path) = @_;
	%Table = ();
	@SameName = ();
	AddFiles($path);
	for my $item (@SameName) {
		$item =~ m|/([^/]*)$|;
		if (defined($Table{$1})) {
			push(@{$Sames{$1}}, $Table{$1});
			delete $Table{$1};
		}
		push(@{$Sames{$1}}, $item);
	}
}
MakeTable($Obj1) if $DirMode;

sub TryDecode($) {
	my ($text) = @_;
	$text = Encode::encode("iso-latin-1", $text);
	for (@TryEncodings) {
		undef $@;
		my $out = eval { Encode::decode($_, $text, Encode::FB_CROAK); };
		return $out if (not $@);
	}
	return "";
}

sub Compare($$) {
	my ($file1, $file2) = @_;
	print "%%%%%%%%%%%%%%%%\n/usr/bin/diff -a '$file1' '$file2'\n";
	my ($temp1, $temp2) = (NewLine($file1), NewLine($file2));
	# can't control `` encoding. use file instead.
	my $tmpout = NewFile($file2);
	`/usr/bin/diff -a '$temp1' '$temp2' >& '$tmpout'`;
	open(IN, "<:encoding(:iso-latin-1)", $tmpout) or die "'$tmpout': $! ";
	my $diff = join("", <IN>);
	close(IN);
	unlink $tmpout;
	unlink $temp1 unless ($temp1 eq $file1);
	unlink $temp2 unless ($temp2 eq $file2);
	return unless (length($diff));
	my $decoded = TryDecode($diff);
	if ($decoded) {
		binmode(STDOUT, ":utf8");
		print $decoded;
	} else {
		warn "bad encoding: '$file1'\n";
		binmode(STDOUT, ":encoding(iso-latin-1)");
		print $diff;
	}
}

my @SameResult = ();
my @NotFound = ();
sub ParseFiles($);
sub ParseFiles($) {
	my ($path) = @_;
	if (-d $path) {
		opendir(DIR, $path) or die $!;
		my @items = readdir(DIR);
		closedir(DIR);
		for (@items) { utf8::decode($_); }
		for my $item (@items) {
			next if ($item =~ /^\./);
			next if (-l "$path/$item");
			ParseFiles("$path/$item");
		}
	} else {
		$path =~ m|/([^/]*)$|;
		if (defined($Table{$1})) {
			Compare($Table{$1}, $path);
			delete $Table{$1};
		} elsif (defined($Sames{$1})) {
			push(@SameResult, $path);
		} else {
			push(@NotFound, $path);
		}
	}
}

if ($DirMode) { ParseFiles($Obj2); }
else { Compare($Obj1, $Obj2); }

if (@SameResult) {
	print "%%%% multiple files of same name %%%%\n";
	for my $path (@SameResult) {
		print "(M2) $path\n";
		my ($name) = $path =~ m|/([^/]*)$|;
		for my $obj (@{$Sames{$name}}) {
			print "(M1) $obj\n";
		}
	}
}

if (%Table) {
	print "%%%% files (1) not found in (2) %%%%\n";
	for my $key (keys %Table) {
		print "(F1) $Table{$key}\n";
	}
}

if (@NotFound) {
	print "%%%% files (2) not found in (1) %%%%\n";
	for my $path (@NotFound) {
		print "(F2) $path\n";
	}
}
