#! /usr/bin/perl -w
use strict;
use utf8;

# 他の形式を SJIS 形式の JTeXEdit 文書に直す perl スクリプト

use Encode;
use Foundation;
# man PerlObjCBridge

my @encodings = ('EUC-JP', 'ISO-2022-JP', 'JIS0208', 'SJIS', 'UTF8');

sub Parse($) {
	my ($file) = @_;
	open(IN, "<:bytes", $file) or die "Can't open-read $!";
	my $data = join('', <IN>);
	close(IN);
	for (@encodings) {
		undef $@;
		my $text = eval { Encode::decode($_, $data, Encode::FB_CROAK); };
		next if ($@);
		$text =~ tr/\x{00a5}\x{203e}/\\~/;
			# japanese-yen, overline
		open(OUT, ">:encoding(shift_jis)", "$file-SJIS") or die $!;
		print OUT $text;
		close(OUT);
		return;
	}
	warn "$file は読めませんでした\n";
}

sub SetFileTypes($) {
	my ($path) = @_;
	my $sign = NSNumber->numberWithInt_(unpack("N", 'jtxE'));
	my $type = NSNumber->numberWithInt_(unpack("N", 'TEXT'));
	my $dict = NSMutableDictionary->dictionary;
	$dict->setObject_forKey_($sign, "NSFileHFSCreatorCode");
	$dict->setObject_forKey_($type, "NSFileHFSTypeCode");
	NSFileManager->defaultManager
		->changeFileAttributes_atPath_($dict, $path);
}

while (my $file = shift) {
	Parse($file);
	SetFileTypes($file);
}
