#!/usr/bin/perl -w
use strict;
use utf8;

use Foundation;
# man PerlObjCBridge

# sub examples that use Foundation

# return (creator, file_type)
sub GetHFSFileTypes($) {
	my ($path) = @_;
	my $dict = NSFileManager->defaultManager
		->attributesOfItemAtPath_error_($path, undef);
	my $sign = $dict->objectForKey_("NSFileHFSCreatorCode")->unsignedIntValue;
	$sign = $sign ? pack("N", $sign) : '';
	my $type = $dict->objectForKey_("NSFileHFSTypeCode")->unsignedIntValue;
	$type = $type ? pack("N", $type) : '';
	return ($sign, $type);
}

# set (creator = 'jtxE', file_type = 'utf8')
sub SetHFSFileTypes($) {
	my ($path) = @_;
	my $dict = NSMutableDictionary->dictionary;
	my $sign = NSNumber->numberWithInt_(unpack("N", 'jtxE'));
	$dict->setObject_forKey_($sign, "NSFileHFSCreatorCode");
	my $type = NSNumber->numberWithInt_(unpack("N", 'utf8'));
	$dict->setObject_forKey_($type, "NSFileHFSTypeCode");
	NSFileManager->defaultManager
		->changeFileAttributes_atPath_($dict, $path);
}

# return (data_size, rsrc_size)
sub GetFileSizes($)
{
	my ($path) = @_;
	my $dict = NSFileManager->defaultManager
		->attributesOfItemAtPath_error_($path, undef);
	my $filesize = $dict->objectForKey_("NSFileSize")->unsignedLongLongValue;
	my $rsrcsize = 0;
	$dict = NSFileManager->defaultManager
		->attributesOfItemAtPath_error_("$path/rsrc", undef);
	$rsrcsize = $dict->objectForKey_("NSFileSize")->unsignedLongLongValue if ($dict);
	return ($filesize, $rsrcsize);
}

# delete rsrc_fork
sub DeleteRsrcFork($)
{
	my ($path) = @_;
	return '' unless (-e "$path/rsrc");
	open(OUT, ">$path/rsrc") or return '';
	print OUT "";
	return 1;
}

# return creation, modification dates
# output format: YYYY-MM-DD HH:MM:SS ([+-] HHMM)
sub GetFileDates($) {
	my ($path) = @_;
	my $dict = NSFileManager->defaultManager
		->attributesOfItemAtPath_error_($path, undef);
	my $cdate = $dict->objectForKey_("NSFileCreationDate")
		->description->UTF8String;
	my $mdate = $dict->objectForKey_("NSFileModificationDate")
		->description->UTF8String;
	return ($cdate, $mdate);
}

# set creation, modification dates
# input format: YYYY-MM-DD HH:MM:SS
sub SetFileDates($$$) {
	my ($path, $cdate, $mdate) = @_;
	my $now = NSDate->date->description->UTF8String;
	my ($zone) = $now =~ /([+-]\d{4})$/;
	my $dict = NSMutableDictionary->dictionary;
	$cdate = NSDate->dateWithString_("$cdate $zone");
	$dict->setObject_forKey_($cdate, "NSFileCreationDate");
	$mdate = NSDate->dateWithString_("$mdate $zone");
	$dict->setObject_forKey_($mdate, "NSFileModificationDate");
	NSFileManager->defaultManager
		->changeFileAttributes_atPath_($dict, $path);
}

# return creator if (hfs_file_type eq 'alis')
sub IsAliasFile($)
{
	my ($path) = @_;
	my ($sign, $type) = GetHFSFileTypes($path);
	return ($type eq 'alis') ? $sign : undef;
}
