#!/usr/bin/perl -w
use utf8;
#
# これは tex の文書から JTeXEdit 用の KeyWords を生成するための
# perl 用のプログラムです．
#
# 文書又はフォルダ ABC をこの画面にドラッグすると
# KeyWords フォルダに ABC.words という文書を生成します．
# 複数の文書をドラッグすれば個別に処理します．
# 結果として重複する単語があってもかまいません．
#
# 2015-04-01 小磯憲史

# ver 6.2c02
#	.cls も処理
# ver 6.2
#	@ を含む文字列を排除
# ver 5.1
#	0x7F 以上の文字を含む文字列を排除
# ver 2.0
#	OSX 用
# ver 1.5
#   MacJPerl での日本語に対応
# ver 1.3
#   'Icon\n' を無視する．
#   MacJPerl にも対応．
# ver 1.2:
#   @ は通常の文字に見做す．
# ver 1.1:
#   words と同じフォルダに生成する．
#   重複する文書名には番号をふる．
#   フォルダも受け付ける．
#   % 以下を無視．

use strict;
use Encode;
use Foundation;
# man PerlObjCBridge

my ($TrgDir) = $0 =~ m|^(.*)/[^/]*$|;
my %KeyWords;
#my %Ext2Type; # 1 = tex, 2 = not

#my %Rejects;
#for (qw/1 2 3 aux bat bib bug cmap cnf css ctl ctx dat default dtd eps fo hyp jpg kc lfg log lmx lua lus mkii mkiv mkvi nanum pat pdf png ps rlx rme rng sample sfd svg tab us xml xsd xsl xyp README/)
#{ $Rejects{$_} = 1; }
my %Parses;
for (qw/cls ltx ini ins ori sti sty tex/)
{ $Parses{$_} = 1; }

sub ConvertDataToString($) {
	my ($data) = @_;
	for (qw/shift_jis utf8 iso-latin1/) {
		undef $@;
		my $text = eval { Encode::decode($_, $data, Encode::FB_CROAK); };
		return $text unless ($@);
	}
	return undef;
}

sub DoFile($) {
	my ($in) = @_;
	return unless (-T $in);
	return if ($in =~ m|/Icon\r$|);
	my ($ext) = $in =~ m|\.([^./]*)$| or return;
	return unless ($Parses{$ext});
	my $count = 0;
#	print "scan: $in\n";
	open(IN, "<:bytes", $in) or die "Can't open-read $!";
	my $data = join('', <IN>);
	close(IN);
	# SJIS と UTF-8 のみ
	my $text = ConvertDataToString($data);
	unless (defined $text) { warn "Can't read $in"; return; }
	$text =~ tr/\x{00a5}/\\/; # japanese-yen => backslash
	my @lines = split(/\n|\r/, $text);
	my @exceptional = ();
	while (defined (my $line = shift @lines)) {
		while ($line ne '') {
			$line =~ s/^[^\\\%]*//; # 先頭の \, % まで削除
			last if ($line eq '' or $line eq "\\" or $line =~ /^\%/);
			if ($line =~ s/^(\\([a-zA-Z@])+)//) {
				my $tmp = $1;
				unless ($tmp =~ /[@\x7F-\xFF]/) {
					$KeyWords{$1} = 1;
					++$count;
				}
			} else {
				if ($line =~ /^(\\[^a-zA-Z])/) {
					$KeyWords{$1} = 1;
					++$count;
				}
				$line = substr($line, 2);
			}
		}
	}
#	return if ($Parses{$ext});
#	print "first: $in\n" unless ($Ext2Type{$ext});
#	$Ext2Type{$ext} |= ($count ? 1 : 2);
}

sub DoFold($);
sub DoFold($) {
	my ($in) = @_;
	print "dir: $in\n";
	opendir(DIR, $in) or die "Can't opendir $in\n";
	my @items = readdir(DIR);
	closedir DIR;
	for my $item (@items) {
		next if ($item =~ /^\./);
		if (-d "$in/$item") {
			DoFold("$in/$item");
		} else {
			DoFile("$in/$item");
		}
	}
}

sub SetFileTypes($) {
	my ($path) = @_;
	my $sign = NSNumber->numberWithInt_(unpack("N", 'jtxE'));
	my $type = NSNumber->numberWithInt_(unpack("N", 'utf8'));
	my $dict = NSMutableDictionary->dictionary;
	$dict->setObject_forKey_($sign, "NSFileHFSCreatorCode");
	$dict->setObject_forKey_($type, "NSFileHFSTypeCode");
	NSFileManager->defaultManager
		->changeFileAttributes_atPath_($dict, $path);
}

while (my $in = shift @ARGV) {
	$in =~ m|/([^/]*)$|;
	%KeyWords = ();
	my $out = "$TrgDir/$1.words";
	if (-e $out) {
		my $add = 2;
		while (-e "$out$add") { ++$add; }
		$out .= $add;
	}
	if (-d $in) {
		DoFold($in);
	} else {
		DoFile($in);
	}
	open OUT, ">$out" or die "Can't open-write $out\n";
	foreach (sort keys %KeyWords) {
		print OUT "$_\r"; # \cr
	}
	close OUT;
	# JTeXEdit 文書として保存．
	SetFileTypes($out);
}

print "**** done ****\n";

#print "**** type ****\n";
#my (@Type_TeX, @Type_Not, @Type_Both);
#for (sort keys %Ext2Type) {
#	my $type = $Ext2Type{$_};
#	if ($type == 1) {
#		push(@Type_TeX, $_);
#	} elsif ($type == 2) {
#		push(@Type_Not, $_);
#	} else {
#		push(@Type_Both, $_);
#	}
#}
#print "tex: ", join(", ", @Type_TeX), "\n";
#print "non-tex: ", join(", ", @Type_Not), "\n";
#print "?-tex: ", join(", ", @Type_Both), "\n";
